/*
 *   alligner.c
 *   ftpippo Vers. 0.9.3
 *                 Copyright (C)  2001  Matteo Nastasi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *     CONTACT:    matteo.nastasi@milug.org
 *                 mnastasi@lycosmail.com
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <stdlib.h>
#include <string.h>
#define __USE_XOPEN
#include <time.h>
#include <utime.h>

#include <ftplib.h>

#define MAIN 1
#include "alligner.h"

void usage(char *s)
{
  fprintf(stderr,"\nUSAGE:\n");
  fprintf(stderr,"%s [--nosafe|-n] [--user|-u] [--skip|-s <skipfile>] <host> <user> <passwd> <usrftpid>\n",s);
  fprintf(stderr,"%s [--help|-h]\n\n",s);
}

int main(int argc, char **argv)
{
  netbuf *mp;
  int disp,pars;
  char *nomeskip = NULL;

  for (disp = 1 ; disp < argc && argv[disp][0] == '-' ; disp++) {
    if (!strcmp(argv[disp],"-h") || !strcmp(argv[disp],"--help")) {
      usage(argv[0]);
      exit(0);
    }
    else if (!strcmp(argv[disp],"-n") || !strcmp(argv[disp],"--nosafe")) 
      safe = 0;
    else if (!strncmp(argv[disp],"-s",2) || !strncmp(argv[disp],"--skip",6)) {
      pars = (argv[disp][1] == '-' ? 6 : 2);
      if (argv[disp][pars] != '\0')
	nomeskip = &(argv[disp][pars]);
      else
	nomeskip = argv[++disp];
    }
    else if (!strncmp(argv[disp],"-r",2) || !strncmp(argv[disp],"--hour",6)) {
      pars = (argv[disp][1] == '-' ? 6 : 2);
      if (argv[disp][pars] != '\0')
	deltam = 3600L * atol(&(argv[disp][pars]));
      else
	deltam = 3600L * atol(argv[++disp]);
    }
    else {
      usage(argv[0]);
      exit(1);
    }      
  }
  if (argc - disp != 4) {
    usage(argv[0]);
    exit(1);
  }
  deltat = delocal();
  
  if (nomeskip) {
    if (!loadskipdir(nomeskip))
      exit (10);
  }

  FtpInit();
  if (!FtpConnect(argv[disp], &mp))
    exit (1);
  if (!FtpLogin(argv[disp+1],argv[disp+2], mp))
    exit (2);

  overdir("",mp,argv[disp+3]);
  FtpQuit(mp);

  if (nomeskip) 
    freeskipdir();

  exit(0);
  return (0);
}
