/*
 *   common.c
 *   ftpippo Vers. 0.9.3
 *                 Copyright (C)  2001  Matteo Nastasi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *     CONTACT:    matteo.nastasi@milug.org
 *                 mnastasi@lycosmail.com
 *
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <stdlib.h>
#include <string.h>
#define __USE_XOPEN
#include <time.h>
#include <utime.h>

#include <ftplib.h>

#include "alligner.h"

#define DEBUG 1 

int loadskipdir(char *nome)
{
  char bf[512];
  int n;
  FILE *fp;
  
  if ((fp = fopen(nome,"r")) == NULL)
    return (0);
  
  for ( ; fgets(bf,512,fp) != NULL ; nskip++) {
    n = strlen(bf) - 1;
    if (bf[n] == '\n')
      bf[n] = '\0';
    if ((nskip % NBLKSKIP) == 0) {
      n = (nskip / NBLKSKIP)+1;
      if (( skip = realloc((void *)skip,n*(NBLKSKIP*sizeof(char *)))) == NULL)
	return (0);
    }
#ifdef DEBUG 
    fprintf(stderr,"LOAD [%s]\n",bf);
#endif
    skip[nskip] = strdup(bf);
  }
  
  fclose(fp);
  return (1);
}

int findskipdir(char *ss)
{
  int i;
  char *s;

  if (ss[0] == '/')
    s = &(ss[1]);
  else if (!strncmp(ss,"./",2))
    s = &(ss[2]);
  
  if (!skip)
    return (0);

#ifdef DEBUG 
    fprintf(stderr,"FIND [%s] [%s]\n",ss, s);
#endif

  for (i = 0 ; i< nskip ; i++)
    if (!strcmp(s,skip[i]))
      return (1);
       
  return (0);
}

/* #undef DEBUG  */

void freeskipdir()
{
  int i;
  
  if (skip) {
    for (i = 0 ; i<nskip ; i++)
      free(skip[i]);
    free(skip);
  }
}

int continuo()
{
  char bf[256];

  fprintf(stderr,"Continue (y/s[kip]/n): ");
  fgets(bf,256,stdin);
  if (bf[0] == 'y' || bf[0] == 'Y')
    return 1;
  else if (bf[0] == 's' || bf[0] == 'S')
    return 0;
  else 
    exit (0);
}

char *relfile(char *ss)
{
  static char s[2048];
  char *p;

  p = s;
  if (*ss == '\0' || *ss == '/') 
    *p++ = '.';
  strcpy(p,ss);

  return (s);
}
    
int aggiornalocdata(char *bf,char *data)
{
  time_t newtim;
  struct tm newtm;
  struct stat newst;
  struct utimbuf newut;

  char *p,zoo[512];
fprintf(stderr,"AGGIORLOCDAT [%s]\n",data);
  p = strptime(data,"%Y%m%d%H%M%S",&newtm);
  if (p == NULL || *p != '\0')
    return (0);
  newtim = mktime(&newtm);

  strftime(zoo,16,"%Y%m%d%H%M%S",localtime(&newtim));
  /* NON HO TROVATO DI MEGLIO PER METTERE A POSTO STA LOCAL DATE */
  newtim -= (time_t)(deltat + deltam);
  lstat(relfile(bf),&newst);
  newut.actime = newst.st_atime;
  newut.modtime = newtim;
  utime(relfile(bf),&newut);
  return (1);
}

int dellistel(elenco *el,int *n,int quale)
{
  int i;

  if (quale >= *n)
    return (0);
  free(el[quale].nome);
  for (i = quale;i < (*n)-1 ; i++)
    memcpy(&(el[i]),&(el[i+1]),sizeof(elenco));
  (*n)--;
  return (1);
}
long delocal()
{
  time_t curr,gmt,loc;

  curr = time(NULL);
  gmt = mktime(gmtime(&curr));
  loc = mktime(localtime(&curr));


  return ((long)difftime(gmt,loc));
}

char *nchar(int n)
{
  static char bf[512];
  int i;

  for (i=0 ; i<n ; i++)
    bf[i] = ' ';
  bf[i] = '\0';
  return (bf);
}

int freeelenco(elenco **ele,int n)
{
  int i;

  for (i = 0 ; i<n ; i++)
    free((*ele)[i].nome);
  free(*ele);
  *ele = NULL;

  return 1;
}

int scannadir(elenco **eledir,int *ndir,elenco **elefil,int *nfil,char *ss)
{  
  DIR *dir;
  struct dirent *pd;
  struct stat ps;
  char bf[2048+16], *s, *p;

  *ndir = 0; *nfil = 0;
  s = relfile(ss);
  if ((dir = opendir(s)) == NULL)
    return (1);
  while((pd = readdir(dir)) != NULL) {
    if (!strcmp(pd->d_name,".") ||
	!strcmp(pd->d_name,".."))
      continue;
    sprintf(bf,"%s/%s",s,pd->d_name);
    if (findskipdir(bf))
      continue;
    if (lstat(bf,&ps))
      return (2);
    if (S_ISLNK(ps.st_mode)) 
      continue;
    else if (S_ISDIR(ps.st_mode)) {
      if ((*eledir = (elenco *)realloc(*eledir,((*ndir)+1) * sizeof(elenco))) == NULL)
	return (3);      
      if ((p = strrchr(bf,'/')) != NULL)
	(*eledir)[*ndir].nome = strdup(++p);
      else
	(*eledir)[*ndir].nome = strdup(bf);
      (*ndir)++;
    }
    else if (S_ISREG(ps.st_mode)) {
      if ((*elefil = realloc(*elefil,((*nfil)+1) * sizeof(elenco))) == NULL)
	return (3);
      if ((p = strrchr(bf,'/')) != NULL)
	(*elefil)[*nfil].nome = strdup(++p);
      else
	(*elefil)[*nfil].nome = strdup(bf);
      strftime((*elefil)[*nfil].data,16,"%Y%m%d%H%M%S",gmtime(&ps.st_mtime));
#ifdef DEBUG
      printf("NOME [%s] [%s]\n",(*elefil)[*nfil].nome,(*elefil)[*nfil].data);
#endif
      (*elefil)[*nfil].data[14] = '\0';
      (*nfil)++;
    }
    else {
      return(3);
    }
  }
  closedir(dir);
  return (0);
  }



int localoverdir(char *s) 
{
  elenco *edir = NULL, *efil = NULL;
  int ndir,nfil;
  int i,ret;

  if ((ret = scannadir(&edir,&ndir,&efil,&nfil,s)) != 0)
    return (ret);
  for (i = 0 ; i<ndir ; i++)
    printf("DIR %s\n",edir[i].nome);
  for (i = 0 ; i<nfil ; i++)
    printf("FIL %s (%s)\n",efil[i].nome,efil[i].data);

  for (i = 0 ; i<ndir ; i++) {
    localoverdir(edir[i].nome);
  }
  freeelenco(&edir,ndir);
  freeelenco(&efil,nfil);
  return (0);
}

char *trovapezzo(char *s,int n,int last)
{
  int st;
  static char bf[2048];

  for (st = 0 ; st < (n*2) && *s != '\0' ;) {
    if (st % 2 == 0) {
      if (*s == ' ')
	st++;
    }
    else {
      if (*s != ' ')
	st++;
    }
    s++;
  }
  if (st != (n*2))
    return (NULL);
  else {   
    s--;
    if (last) {
      for (st = 0 ; *s != '\n' && *s != '\0' ; st++, s++)
	bf[st] = *s;
      bf[st] = '\0';
    }
    else {
      for (st = 0 ; *s != ' ' && *s != '\0' ; st++, s++)
	bf[st] = *s;
      bf[st] = '\0';
    }
    return (bf);
  }
}

/* #define DEBUG 1 */
int scannaftp(elenco **eledir,int *ndir,elenco **elefil,int *nfil,
	      netbuf *mp,char *s,char *user,char *pref)
{  
  FILE *fp;
  char bf[2048+16],paz[2048];

  *ndir = 0; *nfil = 0;
  sprintf(bf,"%s%s",pref,s);
#ifdef DEBUG
  fprintf(stderr,"DELFTPDIR [%s]\n",bf);
#endif
  FtpDir("/var/tmp/alligner.txt",bf,mp);
  if ((fp = fopen("/var/tmp/alligner.txt","r")) == NULL)
    return (1);
  while (fgets(bf,2048,fp) != NULL) {
    if (!memcmp(bf,"total",5))
      continue;
    if (guser && strcmp(trovapezzo(bf,2,0),user))
      continue;
    if (bf[0] == 'd') {
      /* DIRECTORY */
      if (!strcmp(trovapezzo(bf,8,1),".") || 
	  !strcmp(trovapezzo(bf,8,1),".."))
	continue;
      if ((*eledir = (elenco *)realloc(*eledir,((*ndir)+1) * sizeof(elenco))) == NULL)
	return (3);
      (*eledir)[*ndir].nome = strdup(trovapezzo(bf,8,1));
      (*ndir)++;
    }
    else if (bf[0] == '-') {
      if ((*elefil = realloc(*elefil,((*nfil)+1) * sizeof(elenco))) == NULL)
	return (4);
      (*elefil)[*nfil].nome = strdup(trovapezzo(bf,8,1));
      sprintf(paz,"%s/%s",s,(*elefil)[*nfil].nome);
      if (!FtpModDate(paz,(*elefil)[*nfil].data,16,mp))
	return (5);
      (*elefil)[*nfil].data[14] = '\0';
      (*nfil)++;
    }
    else
      continue;
  }
  return (0);
}
/* #undef DEBUG */

int overdir(char *s,netbuf *mp,char *user) 
{
  elenco *edirftp = NULL, *efilftp = NULL;
  int ndirftp,nfilftp;
  elenco *edirloc = NULL, *efilloc = NULL;
  char bf[4096];
  int ndirloc,nfilloc;
  int i,e,ret;
  int remm;

  printf("PATH %s\n",s);
  if ((ret = scannaftp(&edirftp,&ndirftp,&efilftp,&nfilftp,mp,s,user,"")) != 0)
    return (ret);
  if (ndirftp == 0 && nfilftp == 0) {
     if ((ret = scannaftp(&edirftp,&ndirftp,&efilftp,&nfilftp,mp,s,user,"./")) != 0)
        return (ret);
     }
  if ((ret = scannadir(&edirloc,&ndirloc,&efilloc,&nfilloc,s)) != 0)
    return (ret);
#ifdef DEBUG
  for (i = 0 ; i<ndirftp ; i++)
    printf("DIRFTP %s\n",edirftp[i].nome);
  for (i = 0 ; i<nfilftp ; i++)
    printf("FILFTP %s (%s)\n",efilftp[i].nome,efilftp[i].data);
  for (i = 0 ; i<ndirloc ; i++)
    printf("DIRLOC %s\n",edirloc[i].nome);
  for (i = 0 ; i<nfilloc ; i++)
    printf("FILLOC %s (%s)\n",efilloc[i].nome,efilloc[i].data);
#endif
  for (i = 0 ; i<ndirftp ; i++) {
    sprintf(bf,"%s/%s",s,edirftp[i].nome);
    if (findskipdir(bf))
      continue;
    for (e = 0 ; e<ndirloc ; e++) 
      if (!strcmp(edirftp[i].nome,edirloc[e].nome)) 
	break;
    if (e < ndirloc) { /* dir trovata, rimossa dalla lista */
      dellistel(edirloc,&ndirloc,e);
    }
    else { /* dir non trovata, la crea */
      if (mkdir(relfile(bf),0755))
	return (102);
    }
  }
  for (i = 0 ; i<nfilftp ; i++) {
    sprintf(bf,"%s/%s",s,efilftp[i].nome);
    if (findskipdir(bf))
      continue;
    for (e = 0 ; e<nfilloc ; e++) 
      if (!strcmp(efilftp[i].nome,efilloc[e].nome)) 
	break;
    if (e < nfilloc) { /* file trovato verifica data e lo toglie dalla lista */
#ifdef DEBUG
      fprintf(stderr,"DATA FTP %s, LOCAL %s\n",efilftp[i].data,efilloc[e].data);
#endif
      if (strcmp(efilftp[i].data,efilloc[e].data) > 0) {
	printf("GET %s\n",bf);
	if (!FtpGet(relfile(bf),bf,FTPLIB_IMAGE,mp))
	  return (100);
	aggiornalocdata(bf,efilftp[i].data);
      }
      dellistel(efilloc,&nfilloc,e);
    }
    else { /* file non esiste lo crea e gli setta la data */
      printf("GET %s\n",bf);
      if (!FtpGet(relfile(bf),bf,FTPLIB_IMAGE,mp))
	return (101);
      /* aggiorna la data del file */
      aggiornalocdata(bf,efilftp[i].data);
    }
  }
  /* rimuove localmente file e dir non cancellati dalle
     liste locali */
  for (i = 0 ; i<nfilloc ; i++) {
    if (strcmp("",s))
       sprintf(bf,"%s/%s",s,efilloc[i].nome);
    else
       sprintf(bf,"%s",efilloc[i].nome);
    if (findskipdir(bf))
      continue;
    printf("LOCDELFIL %s\n",relfile(bf));
    if (safe)
      remm = continuo();
    if (remm)
       unlink(relfile(bf));
    }
  for (i = 0 ; i<ndirloc ; i++) {
    if (strcmp("",s))
       sprintf(bf,"%s/%s",s,edirloc[i].nome);
    else
       sprintf(bf,"%s",edirloc[i].nome);
    if (findskipdir(bf))
      continue;
    printf("LOCDELDIR %s\n",relfile(bf));
    if (safe)
      remm = continuo();
    if (remm)
       rmlocdir(bf);    
    }

  for (i = 0 ; i<ndirftp ; i++) {
    sprintf(bf,"%s/%s",s,edirftp[i].nome);
    if (findskipdir(bf))
      continue;
    overdir(bf,mp,user);
  }
  freeelenco(&edirftp,ndirftp);
  freeelenco(&efilftp,nfilftp);
  freeelenco(&edirloc,ndirloc);
  freeelenco(&efilloc,nfilloc);

  return (0);
}



int overftp(char *s,netbuf *mp,char *user) 
{
  elenco *edirftp = NULL, *efilftp = NULL;
  int ndirftp,nfilftp;
  elenco *edirloc = NULL, *efilloc = NULL;
  char bf[4096],newdata[16];
  int ndirloc,nfilloc;
  int i,e,ret;
  int remm;

  printf("FTPPATH %s\n",s);
  if ((ret = scannaftp(&edirftp,&ndirftp,&efilftp,&nfilftp,mp,s,user,"")) != 0)
    return (ret);
  if (ndirftp == 0 && nfilftp == 0) {
     if ((ret = scannaftp(&edirftp,&ndirftp,&efilftp,&nfilftp,mp,s,user,"./")) != 0)
        return (ret);
     }
  if ((ret = scannadir(&edirloc,&ndirloc,&efilloc,&nfilloc,s)) != 0)
    return (ret);
#ifdef DEBUG
  for (i = 0 ; i<ndirftp ; i++)
    printf("DIRFTP %s\n",edirftp[i].nome);
  for (i = 0 ; i<nfilftp ; i++)
    printf("FILFTP %s (%s)\n",efilftp[i].nome,efilftp[i].data);
  for (i = 0 ; i<ndirloc ; i++)
    printf("DIRLOC %s\n",edirloc[i].nome);
  for (i = 0 ; i<nfilloc ; i++)
    printf("FILLOC %s (%s)\n",efilloc[i].nome,efilloc[i].data);
#endif
  for (e = 0 ; e<ndirloc ; e++) {
    sprintf(bf,"%s/%s",s,edirloc[e].nome);
    if (findskipdir(bf))
      continue;
    for (i = 0 ; i<ndirftp ; i++)
      if (!strcmp(edirftp[i].nome,edirloc[e].nome)) 
	break;
    if (i < ndirftp) { /* dir trovata, rimossa dalla lista */
      /* rimuove dalla lista la dir ftp */
      dellistel(edirftp,&ndirftp,i);
    }
    else { /* dir non trovata, la crea */
      if (!FtpMkdir(bf,mp)) {
	return (102);
      }
    }
  }

  for (e = 0 ; e<nfilloc ; e++) {
    sprintf(bf,"%s/%s",s,efilloc[e].nome);
    if (findskipdir(bf))
      continue;
    for (i = 0 ; i<nfilftp ; i++)
      if (!strcmp(efilftp[i].nome,efilloc[e].nome)) 
	break;
    if (i < nfilftp) { /* file trovato verifica data e lo toglie dalla lista */
#ifdef DEBUG
      fprintf(stderr,"DATA FTP %s, LOCAL %s\n",efilftp[i].data,efilloc[e].data);
#endif
      if (strcmp(efilftp[i].data,efilloc[e].data) < 0) {
	printf("PUT %s\n",bf);
	if (!FtpPut(relfile(bf),bf,FTPLIB_IMAGE,mp))
	  return (100);
	/* aggiorna data file locale */
	if (!FtpModDate(bf,newdata,16,mp))
	  return (5);
	newdata[14] = '\0';
	aggiornalocdata(bf,newdata);
      }
      dellistel(efilftp,&nfilftp,i);
    }
    else { /* file non esiste lo crea e gli setta la data */
      printf("PUT %s\n",bf);
      if (!FtpPut(relfile(bf),bf,FTPLIB_IMAGE,mp))
	return (101);
      /* aggiorna data file locale */
      if (!FtpModDate(bf,newdata,16,mp))
	return (5);
      newdata[14] = '\0';
      aggiornalocdata(bf,newdata);
    }
  }   
  for (i = 0 ; i<nfilftp ; i++) {
    /* rimuove i file che sono ancora nella lista dei file remoti */
    sprintf(bf,"%s/%s",s,efilftp[i].nome);
    if (findskipdir(bf))
      continue;
    fprintf(stderr,"FTPDELFIL %s\n",bf);
    if (safe)
      remm = continuo();
    if (remm) {
       if (!FtpDelete(bf,mp))
          return (103);
    }
  }
  for (i = 0 ; i<ndirftp ; i++) {
    /* rimuove le dir che sono ancora nella lista dei file remoti */
    sprintf(bf,"%s/%s",s,edirftp[i].nome);
    if (findskipdir(bf))
      continue;
    fprintf(stderr,"FTPDELDIR %s\n",bf);
    if (safe)
       remm = continuo();
    if (remm) {
       if ((ret = rmftpdir(bf,mp,user)) != 0) {
         fprintf(stderr,"Uscita in errore %d\n",ret);
         return (ret);
       }
    }
  }
  for (i = 0 ; i<ndirloc ; i++) {
    sprintf(bf,"%s/%s",s,edirloc[i].nome);
    if (findskipdir(bf))
      continue;
    overftp(bf,mp,user);
  }

  freeelenco(&edirftp,ndirftp);
  freeelenco(&efilftp,nfilftp);
  freeelenco(&edirloc,ndirloc);
  freeelenco(&efilloc,nfilloc);

  return (0);
}


int rmlocdir(char *s) 
{
  elenco *edirloc = NULL, *efilloc = NULL;
  int ndirloc,nfilloc;
  char bf[4096];
  int i,ret;

  fprintf(stderr,"LOCDELDIR %s\n",s);

  if ((ret = scannadir(&edirloc,&ndirloc,&efilloc,&nfilloc,s)) != 0)
    return (ret);

#ifdef DEBUG
  for (i = 0 ; i<ndirloc ; i++)
    printf("DELDIRLOC %s\n",edirloc[i].nome);
  for (i = 0 ; i<nfilloc ; i++)
    printf("DELFILLOC %s (%s)\n",efilloc[i].nome,efilloc[i].data);
#endif
  /* -- cancella file nella dir corrente -- */
  for (i = 0 ; i<nfilloc ; i++) {
     if (strcmp(s,""))
        sprintf(bf,"%s/%s",s,efilloc[i].nome);
     else
        sprintf(bf,"%s",efilloc[i].nome);
     fprintf(stderr,"UNLINK %s\n",relfile(bf));
     unlink(relfile(bf));
     }
  freeelenco(&efilloc,nfilloc);
  for (i = 0 ; i<ndirloc ; i++) {
    /* -- ricorre per cancellare i file nelle eventuali subdir -- */
#ifdef DEBUG
    fprintf(stderr,"RECURRM %s/%s\n",s,edirloc[i].nome);
#endif
    sprintf(bf,"%s/%s",s,edirloc[i].nome);
    rmlocdir(bf);
  }
  /* -- rimuove la dir passata come parametro -- */
  rmdir(relfile(s));
  freeelenco(&edirloc,ndirloc);
  return (0);
}

int rmftpdir(char *s,netbuf *mp,char *user)
{
  elenco *edirftp = NULL, *efilftp = NULL;
  int ndirftp,nfilftp;
  char bf[4096];
  int i,ret;

  fprintf(stderr,"FTPDELDIR %s\n",s);

  if ((ret = scannaftp(&edirftp,&ndirftp,&efilftp,&nfilftp,mp,s,user,"")) != 0)
    return (ret);
  if (ndirftp == 0 && nfilftp == 0) {
     if ((ret = scannaftp(&edirftp,&ndirftp,&efilftp,&nfilftp,mp,s,user,"./")) != 0)
        return (ret);
     }

#ifdef DEBUG
  for (i = 0 ; i<ndirftp ; i++)
    printf("DELDIRFTP %s\n",edirftp[i].nome);
  for (i = 0 ; i<nfilftp ; i++)
    printf("DELFILFTP %s (%s)\n",efilftp[i].nome,efilftp[i].data);
#endif
  /* -- cancella file nella dir corrente -- */
  for (i = 0 ; i<nfilftp ; i++) {
     if (strcmp(s,""))
        sprintf(bf,"%s/%s",s,efilftp[i].nome);
     else
        sprintf(bf,"%s",efilftp[i].nome);

     fprintf(stderr,"FTPDEL %s\n",bf);
     if (!FtpDelete(bf,mp))
       return (101);
  }
  freeelenco(&efilftp,nfilftp);
  for (i = 0 ; i<ndirftp ; i++) {
    /* -- ricorre per cancellare i file nelle eventuali subdir -- */
#ifdef DEBUG
    fprintf(stderr,"RECURFTP %s/%s\n",s,edirftp[i].nome);
#endif
    sprintf(bf,"%s/%s",s,edirftp[i].nome);
    rmftpdir(bf,mp,user);
  }
  /* -- rimuove la dir passata come parametro -- */
  if (!FtpRmdir(s,mp))
    return (102);
  freeelenco(&edirftp,ndirftp);
  return (0);
}

